print __name__

import os, re

class Resource_error(Exception):
   def __init__(self, message):
      self.message = message
   def __str__(self):
      return self.message

class Physical_resource(object):
   def __init__(self, relative_path, full_path):
      self.name = relative_path
      self.path = full_path
      try:
         self.file_size = os.path.getsize(self.path)
         self.timestamp = os.path.getmtime(self.path)
      except OSError, msg:
         raise Resource_error(msg)
         
      self.mem_size = self.file_size

class Image_file(Physical_resource):
   def __init__(self, relative_path, full_path):
      super(Image_file, self).__init__(relative_path, full_path)
      self.width = self.height = 50
      self.mem_size = self.file_size + self.width * self.height * 4
      print "Image %s, %d mem" % (full_path, self.mem_size)

dialog_line = re.compile(r'\[(?P<num>\d+)\](?P<text>.*)', re.DOTALL)

def get_text_from_xml_node(node):
   text = []
   for node in node.childNodes:
      if node.nodeType == node.TEXT_NODE:
         text.append(node.data.encode("iso-8859-1"))
   return "".join(text)
   
class Text_file(Physical_resource):
   def __init__(self, relative_path, full_path):
      super(Text_file, self).__init__(relative_path, full_path)
      self.lines = {}
      if "xml" == os.path.splitext(self.path)[1].lower()[1:]:
         self.__load_xml_text()
      else:
         self.__load_plain_text()
      print "Text %s" % (full_path)

   def __load_xml_text(self):
      import xml.dom.minidom
      dom = xml.dom.minidom.parse(self.path)
      root = dom.getElementsByTagName("dialog_lines")
      if len(root) != 1:
         raise Resource_error("El fichero XML es invlido")
      for line in root[0].getElementsByTagName("line"):
         number = line.getAttribute("number")
         if not number:
            continue

         self.lines[int(line.getAttribute("number"))] = get_text_from_xml_node(line)

   def __load_plain_text(self):
      file = open(self.path, "rt")
      number = -1
      strings = []
      for line in file:
         match = dialog_line.match(line)
         if match:
            if number >= 0:
               self.lines[number] = "".join(strings).rstrip()
            number = int(match.group("num"))
            strings = [match.group("text")]
         else:
            if number >= 0:
               strings.append(line)
      if number >= 0:
         self.lines[number] = "".join(strings).rstrip()
      file.close()

def create_resource(relative_path, full_path):
   ext = os.path.splitext(full_path)[1].lower()[1:]
   try:
      type = factory_function[ext]
      return type(relative_path, full_path)
   except KeyError:
      return None

# one time module initialisation
factory_function = {}
for t in ["bmp", "pcx", "tga", "lbm", "jpg", "jpeg", "png", "gif"]:
   factory_function[t] = Image_file
for t in ["txt", "xml"]:
   factory_function[t] = Text_file
#for t in ["wav", "ogg", "midi", "mid"]:
#   factory_function[t] = None
